@@ -11,10 +11,10 @@
 #include "object.h"
 #include "rectangle.h"
 #include "regexstr.h"
-#include "theme.h"
 #include "x11-types.h"
 
 class Decoration;
+class ResizeAction;
 class DecTriple;
 class Ewmh;
 class FrameLeaf;
@@ -23,7 +23,10 @@ class HSTag;
 class Monitor;
 class Settings;
 class ClientManager;
+class Theme;
+class DecorationScheme;
 class XConnection;
+enum class ThemeType;
 
 class Client : public Object {
 public:
@@ -38,6 +41,7 @@ public:
     Slice* slice = {};
     bool        ewmhfullscreen_ = false; // ewmh fullscreen state
     bool        neverfocus_ = false; // do not give the focus via XSetInputFocus
+    Attribute_<bool> decorated_;
     Attribute_<bool> visible_;
     bool        dragged_ = false;  // if this client is dragged currently
     int         ignore_unmaps_ = 0;  // Ignore one unmap for each reparenting
@@ -45,7 +49,7 @@ public:
                                 // notify event
     //! the last time when minimized_ was changed (with discrete time ticks).
     long long int minimizedLastChange_ = 0;
-    // for size hints
+    // for size hints; 0 means 'unset'
     float mina_ = 0;
     float maxa_ = 0;
     int basew_ = 0;
@@ -62,9 +66,11 @@ public:
 
     // attributes:
     Attribute_<bool> urgent_;
+    bool x11urgent_ = false;
     Attribute_<bool> floating_;
     Attribute_<bool> fullscreen_;
     Attribute_<bool> minimized_;
+    Attribute_<bool> floating_effectively_;
     Attribute_<std::string> title_;  // or also called window title; this is never NULL
     DynAttribute_<std::string> tag_str_;
     DynChild_<FrameLeaf> parent_frame_;
@@ -81,6 +87,7 @@ public:
     DynAttribute_<std::string> window_class_;
     DynAttribute_<std::string> window_instance_;
     Attribute_<Rectangle> content_geometry_;
+    DynAttribute_<Rectangle> decoration_geometry_;
 
 public:
     void init_from_X();
@@ -97,33 +104,28 @@ public:
     Window decorationWindow();
     friend void mouse_function_resize(XMotionEvent* me);
 
-    // other member functions
-    void window_focus();
-    void window_unfocus();
-    static void window_unfocus_last();
-
     void fuzzy_fix_initial_position();
 
     Rectangle outer_floating_rect();
 
     void setup_border(bool focused);
-    void resize_tiling(Rectangle rect, bool isFocused, bool minimalDecoration);
+    void resize_tiling(Rectangle rect, bool isFocused, bool minimalDecoration, std::vector<Client*> tabs);
     void resize_floating(Monitor* m, bool isFocused);
     void resize_fullscreen(Rectangle m, bool isFocused);
     bool is_client_floated();
     void set_urgent(bool state);
-    void update_wm_hints();
+    void readWmHints(bool forceNotUrgent = false);
     void update_title();
     void raise();
     void lower();
 
     void send_configure(bool force);
-    bool applysizehints(int *w, int *h);
+    bool applysizehints(int* w, int* h, bool force = false);
     void updatesizehints();
+    ResizeAction possibleResizeActions();
 
     void set_visible(bool visible);
 
-    void set_urgent_force(bool state);
     void requestClose(); //! ask the client to close
 
     void clear_properties();
@@ -132,6 +134,11 @@ public:
     void updateEwmhState();
 private:
     void floatingGeometryChanged();
+    void urgencyAttributeChanged(bool state);
+    void fixParentWindow(bool decorated);
+    void redraw();
+    void redrawRelevantTabBars();
+    Rectangle decorationGeometry();
     std::string getWindowClass();
     std::string getWindowInstance();
     std::string triggerRelayoutMonitor();
@@ -145,7 +152,8 @@ private:
     XConnection& X_;
     std::string tagName();
     const DecTriple& getDecTriple();
-    Theme::Type mostRecentThemeType;
+    const DecorationScheme& getDecorationScheme(bool focused);
+    ThemeType mostRecentThemeType;
 };
 
 
