@@ -45,6 +45,8 @@ ClientManager::ClientManager()
     dragged.setDoc("the object of a client which is currently dragged"
                    " by the mouse, if any. See the documentation of the"
                    " mousebind command for examples.");
+
+    focus.changed().connect(this, &ClientManager::focusedClientChanges);
 }
 
 ClientManager::~ClientManager()
@@ -163,6 +165,7 @@ void ClientManager::add(Client* client)
         this->clientStateChanged.emit(client);
     });
     addChild(client, client->window_id_str);
+    clientAdded.emit(client);
 }
 
 void ClientManager::setDragged(Client* client) {
@@ -205,8 +208,11 @@ Client* ClientManager::manage_client(Window win, bool visible_already, bool forc
     auto stdio = OutputChannels::stdio();
     changes = Root::get()->rules()->evaluateRules(client, stdio, changes);
     if (!changes.manage || force_unmanage) {
-        // map it... just to be sure
-        XMapWindow(X_->display(), win);
+        // if the window becomes unmanaged and wasn't visible before,
+        // then map it.
+        if (!visible_already) {
+            XMapWindow(X_->display(), win);
+        }
         delete client;
         return {};
     }
@@ -361,7 +367,7 @@ void ClientManager::setSimpleClientAttributes(Client* client, const ClientChange
         // do not simply copy the geometry to the attribute
         // but possibly apply the size hints:
         if (client->sizehints_floating_()) {
-            client->applysizehints(&geo.width, &geo.height);
+            client->applysizehints(&geo.width, &geo.height, true);
         }
         client->float_size_ = geo;
     }
@@ -532,6 +538,19 @@ void ClientManager::applyTmpRuleCompletion(Completion& complete)
     }
 }
 
+/**
+ * @brief called whenever clients.focus changes
+ * @param newFocus
+ */
+void ClientManager::focusedClientChanges(Client* newFocus)
+{
+    if (newFocus) {
+        hook_emit({"focus_changed", newFocus->window_id_str(), newFocus->title_()});
+    } else {
+        hook_emit({"focus_changed", "0x0", ""});
+    }
+}
+
 void ClientManager::unmap_notify(Window win) {
     auto client = this->client(win);
     if (!client) {
