@@ -51,6 +51,7 @@ public:
 
     Signal_<HSTag*> needsRelayout;
     Signal_<Client*> clientStateChanged; //! floating or minimized changed
+    Signal_<Client*> clientAdded;
     Link_<Client> focus;
     Link_<Client> dragged;
 
@@ -72,6 +73,7 @@ public:
     void applyTmpRuleCompletion(Completion& complete);
 
 protected:
+    void focusedClientChanges(Client* newFocus);
     int clientSetAttribute(std::string attribute, Input input, Output output);
     void setSimpleClientAttributes(Client* client, const ClientChanges& changes);
     Theme* theme;
