@@ -4,6 +4,7 @@
 
 #include "globals.h"
 
+using std::function;
 using std::make_shared;
 using std::shared_ptr;
 using std::vector;
@@ -29,12 +30,6 @@ void DesktopWindow::registerDesktop(Window win) {
     windows.push_back(dw);
 }
 
-void DesktopWindow::lowerDesktopWindows() {
-    for (auto dw : windows) {
-        XLowerWindow(g_display, dw->win_);
-    }
-}
-
 void DesktopWindow::unregisterDesktop(Window win) {
     windows.erase(std::remove_if(
                    windows.begin(), windows.end(),
@@ -43,3 +38,10 @@ void DesktopWindow::unregisterDesktop(Window win) {
                    }),
                   windows.end());
 }
+
+void DesktopWindow::foreachDesktopWindow(function<void (DesktopWindow&)> loopbody)
+{
+    for (auto& dw : windows) {
+        loopbody(*dw.get());
+    }
+}
