@@ -2,6 +2,7 @@
 #define __HERBSTLUFT_DESKTOPWINDOW_H_
 
 #include <X11/X.h>
+#include <functional>
 #include <memory>
 #include <vector>
 
@@ -16,8 +17,8 @@ public:
     bool below() const;
     Window window() const;
     static void registerDesktop(Window win);
-    static void lowerDesktopWindows();
     static void unregisterDesktop(Window win);
+    static void foreachDesktopWindow(std::function<void(DesktopWindow&)> loopbody);
 private:
 
     // members:
