@@ -7,6 +7,7 @@
 #include <cstdio>
 
 #include "client.h"
+#include "clientmanager.h"
 #include "globals.h"
 #include "hlwmcommon.h"
 #include "layout.h"
@@ -92,6 +93,8 @@ Ewmh::Ewmh(XConnection& xconnection)
         auto atom = XInternAtom(X_.display(), init.second, False);
         wmatom_[static_cast<size_t>(init.first)] = atom;
     }
+    hlwmFloatingWindow_ = X_.atom("HLWM_FLOATING_WINDOW");
+    hlwmTilingWindow_ = X_.atom("HLWM_TILING_WINDOW");
 
     readInitialEwmhState();
 
@@ -208,6 +211,18 @@ Atom Ewmh::windowManagerSelection()
     return XInternAtom(X_.display(), atomName.c_str(), False);
 }
 
+Atom Ewmh::compositingManagerSelection()
+{
+    // see https://specifications.freedesktop.org/wm-spec/wm-spec-latest.html#idm45381391209264
+    string atomName = "_NET_WM_CM_S" + to_string(X_.screen());
+    return XInternAtom(X_.display(), atomName.c_str(), False);
+}
+
+bool Ewmh::detectCompositingManager()
+{
+    return XGetSelectionOwner(X_.display(), compositingManagerSelection()) != None;
+}
+
 void Ewmh::InitialState::print(FILE *file)
 {
     fprintf(file, "EWMH: %zu desktops:", numberOfDesktops);
@@ -226,6 +241,15 @@ void Ewmh::InitialState::print(FILE *file)
 void Ewmh::injectDependencies(Root* root) {
     root_ = root;
     tags_ = root->tags();
+    // listen to changes regarding floating state:
+    root->clients->clientAdded.connect([this](Client* client) {
+        // on a new client, initially set the x11 property:
+        updateFloatingState(client);
+        // and listen to changes of the floating state:
+        client->floating_effectively_.changed().connect([this,client]() {
+            updateFloatingState(client);
+        });
+    });
 }
 
 void Ewmh::updateAll() {
@@ -374,7 +398,7 @@ void Ewmh::handleClientMessage(XClientMessageEvent* me) {
                 // Focus stealing is not allowed, at least mark the client urgent
                 auto client = Root::common().client(me->window);
                 if (client) {
-                    client->set_urgent(true);
+                    client->urgent_ = true;
                 }
             }
             break;
@@ -421,7 +445,7 @@ void Ewmh::handleClientMessage(XClientMessageEvent* me) {
                 { NetWmStateFullscreen,
                     client->fullscreen_,     [](Client* c, bool state){ c->fullscreen_ = state; } },
                 { NetWmStateDemandsAttention,
-                    client->urgent_,         [](Client* c, bool state){ c->set_urgent(state); } },
+                    client->urgent_,         [](Client* c, bool state){ c->urgent_ = state; } },
             };
 
             /* me->data.l[1] and [2] describe the properties to alter */
@@ -472,7 +496,7 @@ void Ewmh::handleClientMessage(XClientMessageEvent* me) {
                 }
             } else {
                 // anything else is a resize
-                root_->mouse->mouse_initiate_resize(client, {});
+                root_->mouse->mouse_initiate_resize(client, vector<string>());
             }
             break;
         }
@@ -515,6 +539,21 @@ void Ewmh::updateWindowState(Client* client) {
         32, PropModeReplace, (unsigned char *) window_state, count_enabled);
 }
 
+/**
+ * @brief Set an x11 property reporting the client's floating state
+ * @param the client
+ */
+void Ewmh::updateFloatingState(Client* client)
+{
+    if (client->is_client_floated()) {
+        X_.setPropertyCardinal(client->window_, hlwmFloatingWindow_, {1});
+        X_.deleteProperty(client->window_, hlwmTilingWindow_);
+    } else {
+        X_.deleteProperty(client->window_, hlwmFloatingWindow_);
+        X_.setPropertyCardinal(client->window_, hlwmTilingWindow_, {1});
+    }
+}
+
 void Ewmh::clearClientProperties(Window win) {
     // delete ewmh-properties and ICCCM-Properties such that the client knows
     // that he has been unmanaged and now the client is allowed to be mapped
