@@ -94,10 +94,11 @@ void FrameDecoration::render(const FrameDecorationData& data, bool isFocused) {
     }
     int bw = settings->frame_border_width();
 
-    if (settings->smart_frame_surroundings() && !data.hasParent) {
+    if (settings->smart_frame_surroundings() == SmartFrameSurroundings::hide_all
+        && !data.hasParent) {
         bw = 0;
     }
-    Rectangle rect = data.geometry;
+    Rectangle rect = data.contentGeometry;
     XSetWindowBorderWidth(xcon.display(), window, bw);
     XMoveResizeWindow(xcon.display(), window,
                       rect.x - bw,
@@ -123,8 +124,8 @@ void FrameDecoration::render(const FrameDecorationData& data, bool isFocused) {
         }
         for (Client* client : frame_.clients) {
             Rectangle geom = client->dec->last_outer();
-            geom.x -= data.geometry.x;
-            geom.y -= data.geometry.y;
+            geom.x -= data.contentGeometry.x;
+            geom.y -= data.contentGeometry.y;
             holes.push_back(geom);
         }
         window_cut_rect_holes(xcon, window, rect.width, rect.height, holes);
@@ -144,9 +145,31 @@ void FrameDecoration::render(const FrameDecorationData& data, bool isFocused) {
 
 void FrameDecoration::updateVisibility(const FrameDecorationData& data, bool isFocused)
 {
-    bool show = settings->always_show_frame()
-              || data.hasClients
-              || isFocused;
+    bool show = false;
+    bool isRootFrame = frame_.parent_.expired();
+    switch (settings->show_frame_decorations()) {
+    case ShowFrameDecorations::none:
+        show = false;
+        break;
+    case ShowFrameDecorations::nonempty:
+        show = data.hasClients;
+        break;
+    case ShowFrameDecorations::if_empty:
+        show = !data.hasClients;
+        break;
+    case ShowFrameDecorations::focused:
+        show = data.hasClients || isFocused;
+        break;
+    case ShowFrameDecorations::focused_if_multiple:
+        show = data.hasClients || (isFocused && !isRootFrame);
+        break;
+    case ShowFrameDecorations::if_multiple:
+        show = data.hasClients || !isRootFrame;
+        break;
+    case ShowFrameDecorations::all:
+        show = true;
+        break;
+    }
     XConnection& xcon = XConnection::get();
     if (show != visible) {
         visible = show;
