@@ -100,8 +100,7 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
         {"disjoin_rects",  disjoin_rects_command},
         {"list_keybinds",  {keys, &KeyManager::listKeybindsCommand }},
         {"list_padding",   monitors->byFirstArg(&Monitor::list_padding, &Monitor::noComplete) },
-        {"keybind",        {keys, &KeyManager::addKeybindCommand,
-                                  &KeyManager::addKeybindCompletion}},
+        {"keybind",        {keys, &KeyManager::keybindCommand}},
         {"keyunbind",      {keys, &KeyManager::removeKeybindCommand,
                                   &KeyManager::removeKeybindCompletion}},
         {"mousebind",      {mouse, &MouseManager::addMouseBindCommand,
