@@ -13,6 +13,8 @@ class Completion;
 class ClientManager;
 class MonitorManager;
 class MouseDragHandler;
+class ResizeAction;
+class TagManager;
 struct Point2D;
 
 class MouseManager : public Object {
@@ -20,7 +22,7 @@ public:
     MouseManager();
     ~MouseManager();
 
-    void injectDependencies(ClientManager* clients, MonitorManager* monitors);
+    void injectDependencies(ClientManager* clients, TagManager* tags, MonitorManager* monitors);
 
     int addMouseBindCommand(Input input, Output output);
 
@@ -41,7 +43,9 @@ public:
     std::string mouse_initiate_move(Client* client, const std::vector<std::string> &cmd);
     std::string mouse_initiate_zoom(Client* client, const std::vector<std::string> &cmd);
     std::string mouse_initiate_resize(Client* client, const std::vector<std::string> &cmd);
+    std::string mouse_initiate_resize(Client* client, const ResizeAction& resize);
     std::string mouse_call_command(Client* client, const std::vector<std::string> &cmd);
+    ResizeAction resizeAction();
 
 private:
     //! start dragging for the specified client (possibly up to some arguments), and return a error message
@@ -63,13 +67,14 @@ private:
     MouseFunction string2mousefunction(const std::string& name);
 
     //! manually (forward-)declare MouseDragHandler::Constructor as MDC here:
-    typedef std::function<std::shared_ptr<MouseDragHandler>(MonitorManager*, Client*)> MDC;
+    typedef std::function<std::shared_ptr<MouseDragHandler>(MonitorManager*, TagManager*, Client*)> MDC;
     //! start a the drag, and if it does not work out, return an error message
-    std::string mouse_initiate_drag(Client* client, const MDC& createHandler);
+    std::string mouse_initiate_drag(Client* client, const MDC& createHandler, ResizeAction resize);
 
     std::map<std::string, MouseFunction> mouseFunctions_;
     std::shared_ptr<MouseDragHandler> dragHandler_;
     Cursor cursor;
     ClientManager*  clients_;
     MonitorManager*  monitors_;
+    TagManager*  tags_;
 };
