@@ -16,6 +16,30 @@ using std::function;
 using std::string;
 using std::to_string;
 
+template<>
+Finite<SmartFrameSurroundings>::ValueList Finite<SmartFrameSurroundings>::values = ValueListPlain {
+    { SmartFrameSurroundings::hide_all, "hide_all" },
+    { SmartFrameSurroundings::hide_all, "on" },
+    { SmartFrameSurroundings::hide_all, "true" },
+    { SmartFrameSurroundings::hide_all, "1" },
+    { SmartFrameSurroundings::hide_gaps, "hide_gaps" },
+    { SmartFrameSurroundings::off, "off" },
+    { SmartFrameSurroundings::off, "false" },
+    { SmartFrameSurroundings::off, "0" },
+};
+
+template<>
+Finite<ShowFrameDecorations>::ValueList Finite<ShowFrameDecorations>::values = ValueListPlain {
+    { ShowFrameDecorations::all, "all" },
+    { ShowFrameDecorations::focused, "focused" },
+    { ShowFrameDecorations::focused_if_multiple, "focused_if_multiple" },
+    { ShowFrameDecorations::if_multiple, "if_multiple" },
+    { ShowFrameDecorations::nonempty, "nonempty" },
+    { ShowFrameDecorations::if_empty, "if_empty" },
+    { ShowFrameDecorations::none, "none" },
+};
+
+
 Settings* g_settings = nullptr; // the global settings object
 
 Settings::Settings()
@@ -61,6 +85,7 @@ Settings::Settings()
         &frame_normal_opacity,
         &focus_crosses_monitor_boundaries,
         &always_show_frame,
+        &show_frame_decorations,
         &default_direction_external_only,
         &default_frame_layout,
         &focus_follows_mouse,
@@ -112,15 +137,25 @@ Settings::Settings()
         i->changed().connect(&reset_client_colors);
     }
     frame_bg_transparent.setWritable();
-    for (auto i : {&always_show_frame,
-         &gapless_grid,
+    for (auto i : {&gapless_grid,
          &tabbed_max,
-         &smart_frame_surroundings,
          &smart_window_surroundings,
          &raise_on_focus_temporarily}) {
         i->changed().connect(&all_monitors_apply_layout);
     }
+    show_frame_decorations.changed().connect(&all_monitors_apply_layout);
+    smart_frame_surroundings.changed().connect(&all_monitors_apply_layout);
     wmname.changed().connect([]() { Ewmh::get().updateWmName(); });
+    // connect deprecated attribute to new settings:
+    always_show_frame.changedByUser().connect([this](bool alwaysShow) {
+        this->show_frame_decorations =
+                alwaysShow
+                ? ShowFrameDecorations::all
+                : ShowFrameDecorations::focused;
+    });
+    show_frame_decorations.changed().connect([this](ShowFrameDecorations newValue) {
+        this->always_show_frame = newValue == ShowFrameDecorations::all;
+    });
 
     tree_style.setValidator([] (string new_value) {
         if (utf8_string_length(new_value) < 8) {
@@ -164,7 +199,8 @@ Settings::Settings()
     frame_border_inner_color.setDoc("The color of the inner border of a frame.");
     frame_bg_active_color.setDoc("The fill color of a focused frame.");
     frame_bg_normal_color.setDoc("The fill color of an unfocused frame (It is "
-                                 "only visible if always_show_frame is set).");
+                                 "only visible if non-focused frames are configured "
+                                 "to be visible, see \'show_frame_decorations\').");
     frame_bg_transparent.setDoc(
                 "If set, the background of frames are transparent. That means "
                 "a rectangle is cut out from the inner such that only the "
@@ -237,9 +273,19 @@ Settings::Settings()
 
 
     always_show_frame.setDoc(
-                "If set, all frames are displayed. "
-                "If unset, only frames with focus or with "
-                "windows in them are displayed.");
+                "DEPRECATED, use +show_frame_decorations+ instead. Setting "
+                "this corresponds to \'focused\' in \'show_frame_decorations\'."
+                );
+
+    show_frame_decorations.setDoc(
+                "This controls, which frame decorations are shown at all. \n"
+                "- \'none\' shows no frame decorations at all, \n"
+                "- \'nonempty\' shows decorations of frames that have client windows, \n"
+                "- \'if_multiple\' shows decorations on the tags with at least two frames, \n"
+                "- \'if_empty\' shows decorations of frames that have no client windows, \n"
+                "- \'focused\' shows the decoration of focused and nonempty frames, \n"
+                "- \'focused_if_multiple\' shows decorations of focused and non-empty frames on tags with at least two frames."
+                );
 
     frame_active_opacity.setDoc(
                 "Focused frame opacity in percent. Requires a running "
@@ -278,8 +324,11 @@ Settings::Settings()
                 "in the max layout.");
 
     smart_frame_surroundings.setDoc(
-                "If set, frame borders and gaps will be removed when there\'s "
-                "no ambiguity regarding the focused frame.");
+                "If set to \'hide_all\', frame borders and gaps will be removed "
+                "when there is no ambiguity regarding the focused frame. "
+                "If set to \'hide_gaps\', only frame gaps will be removed when "
+                "there is no ambiguity regarding the focused frame. "
+                "Turn \'off\' to always show frame borders and gaps.");
 
     smart_window_surroundings.setDoc(
                 "If set, window borders and gaps will be removed and minimal "
@@ -543,6 +592,3 @@ void Settings::get_complete(Completion& complete) {
         complete.none();
     }
 }
-
-
-
