@@ -1,8 +1,19 @@
 #include "theme.h"
 
+#include "completion.h"
+
 using std::vector;
 using std::string;
 
+template<>
+Finite<TitleWhen>::ValueList Finite<TitleWhen>::values = ValueListPlain {
+    { TitleWhen::always, "always" },
+    { TitleWhen::never, "never" },
+    { TitleWhen::one_tab, "one_tab" },
+    { TitleWhen::multiple_tabs, "multiple_tabs" },
+};
+
+
 Theme::Theme()
     : fullscreen(*this, "fullscreen")
     , tiling(*this, "tiling")
@@ -26,7 +37,7 @@ Theme::Theme()
           "          │                  ╻\n"
           "          │                  │\n"
           "    ┌────╴│╶─────────────────┷─────┐ ⎫ border_width\n"
-          "    │     │      color             │ ⎬ + title_height\n"
+          "    │     │      color             │ ⎬ + title_height + title_depth\n"
           "    │  ┌──┷─────────────────────┐  │ ⎭ + padding_top\n"
           "    │  │====================....│  │\n"
           "    │  │== window content ==....│  │\n"
@@ -37,7 +48,11 @@ Theme::Theme()
           "\n"
           "Setting an attribute of the theme object just propagates the "
           "value to the respective attribute of the +tiling+ and the +floating+ "
-          "object."
+          "object.\n"
+          "If the title area is divided into tabs, then the not selected tabs "
+          "can be styled using the +tab_...+ attributes. If these attributes are "
+          "empty, then the colors are taken from the theme of the client to which "
+          "the tab refers to."
     );
     tiling.setChildDoc(
                 "configures the decoration of tiled clients, setting one of "
@@ -55,7 +70,10 @@ DecorationScheme::DecorationScheme()
     , proxyAttributes_ ({
         &border_width,
         &title_height,
+        &title_depth,
+        &title_when,
         &title_font,
+        &title_align,
         &title_color,
         &border_color,
         &tight_decoration,
@@ -63,6 +81,10 @@ DecorationScheme::DecorationScheme()
         &inner_width,
         &outer_color,
         &outer_width,
+        &tab_color,
+        &tab_outer_color,
+        &tab_outer_width,
+        &tab_title_color,
         &padding_top,
         &padding_right,
         &padding_bottom,
@@ -90,9 +112,56 @@ DecorationScheme::DecorationScheme()
                             "the window decoration or only the window "
                             "contents of tiled clients (requires enabled "
                             "sizehints_tiling)");
+    title_depth.setDoc("the space below the baseline of the window title");
+    title_when.setDoc("when to show the window title: always, never, "
+                      "if the the client is in a tabbed scenario like a max frame (+one_tab+), "
+                      "if there are +multiple_tabs+ to be shown.");
+    title_align.setDoc("the horizontal alignment of the title within the tab "
+                       "or title bar. The value is one of: left, center, right");
+    tab_color.setDoc("if non-empty, the color of non-urgent and unfocused tabs");
+    tab_outer_color.setDoc(
+                "if non-empty, the outer border color of non-urgent and "
+                "unfocused tabs; if empty, the colors are taken from the tab's"
+                "client decoration settings.");
+    tab_outer_width.setDoc("if non-empty, the outer border width of non-urgent and unfocused tabs");
+    tab_title_color.setDoc("if non-empty, the title color of non-urgent and unfocused tabs");
     reset.setDoc("writing this resets all attributes to a default value");
 }
 
+Rectangle DecorationScheme::outline_to_inner_rect(Rectangle rect, size_t tabCount) const {
+    return rect.adjusted(-*border_width, -*border_width)
+            .adjusted(-*padding_left,
+                      -*padding_top - (showTitle(tabCount) ? (*title_height + *title_depth) : 0),
+                      -*padding_right, -*padding_bottom);
+}
+
+/**
+ * @brief whether to show the window titles
+ * @param the number of tabs
+ * @return
+ */
+bool DecorationScheme::showTitle(size_t tabCount) const
+{
+    if (title_height() == 0) {
+        return false;
+    }
+    switch (title_when()) {
+        case TitleWhen::always: return true;
+        case TitleWhen::never: return false;
+        case TitleWhen::one_tab: return tabCount >= 1;
+        case TitleWhen::multiple_tabs: return tabCount >= 2;
+    }
+    return true; // Dead code. But otherwise, gcc complains
+}
+
+Rectangle DecorationScheme::inner_rect_to_outline(Rectangle rect, size_t tabCount) const {
+    return rect.adjusted(*border_width, *border_width)
+            .adjusted(*padding_left,
+                      *padding_top + (showTitle(tabCount) ? (*title_height + *title_depth) : 0),
+                      *padding_right, *padding_bottom);
+}
+
+
 DecTriple::DecTriple()
    : normal(*this, "normal")
    , active(*this, "active")
@@ -134,3 +203,8 @@ void DecorationScheme::makeProxyFor(vector<DecorationScheme*> decs) {
 }
 
 
+template<>
+void Converter<Inherit>::complete(Completion& complete, const Inherit* relativeTo)
+{
+    complete.full("");
+}
