@@ -5,10 +5,53 @@
 
 #include "attribute_.h"
 #include "child.h"
+#include "converter.h"
+#include "either.h"
+#include "finite.h"
 #include "font.h"
 #include "object.h"
 #include "rectangle.h"
 
+
+/**
+ * @brief Criteria when to show the window title
+ */
+enum class TitleWhen {
+    never,
+    always,
+    one_tab,
+    multiple_tabs,
+};
+
+template <>
+struct is_finite<TitleWhen> : std::true_type {};
+template<> Finite<TitleWhen>::ValueList Finite<TitleWhen>::values;
+template<> inline Type Attribute_<TitleWhen>::staticType() { return Type::NAMES; }
+
+class Inherit {
+public:
+    bool operator==(const Inherit& other) { return true; }
+};
+
+template<>
+inline std::string Converter<Inherit>::str(Inherit payload) { return ""; }
+template<>
+inline Inherit Converter<Inherit>::parse(const std::string &payload) {
+    if (payload.empty()) {
+        return {};
+    }
+    throw std::invalid_argument("Use an empty string for inheriting the value.");
+}
+template<> void Converter<Inherit>::complete(Completion& complete, Inherit const* relativeTo);
+
+
+typedef Either<Inherit,Color> MaybeColor;
+typedef Either<Inherit,unsigned long> MaybeULong;
+template<>
+inline Type Attribute_<MaybeColor>::staticType() { return Type::STRING; }
+template<>
+inline Type Attribute_<MaybeULong>::staticType() { return Type::STRING; }
+
 /** The proxy interface
  */
 
@@ -79,10 +122,13 @@ public:
     DynAttribute_<std::string> reset;
     AttributeProxy_<unsigned long>     border_width = {"border_width", 0};
     AttributeProxy_<unsigned long>     title_height = {"title_height", 0};
+    AttributeProxy_<int>           title_depth = {"title_depth", 0};
+    AttributeProxy_<TitleWhen>     title_when = {"title_when", TitleWhen::always};
     AttributeProxy_<Color>   border_color = {"color", {"black"}};
     AttributeProxy_<bool>    tight_decoration = {"tight_decoration", false}; // if set, there is no space between the
                               // decoration and the window content
     AttributeProxy_<HSFont>  title_font = {"title_font", HSFont::fromStr("fixed")};
+    AttributeProxy_<TextAlign> title_align = {"title_align", TextAlign::left};
     AttributeProxy_<Color>   title_color = {"title_color", {"black"}};
     AttributeProxy_<Color>   inner_color = {"inner_color", {"black"}};
     AttributeProxy_<unsigned long>     inner_width = {"inner_width", 0};
@@ -93,11 +139,16 @@ public:
     AttributeProxy_<unsigned long>     padding_bottom = {"padding_bottom", 0}; // additional window border
     AttributeProxy_<unsigned long>     padding_left = {"padding_left", 0};   // additional window border
     AttributeProxy_<Color>   background_color = {"background_color", {"black"}}; // color behind client contents
+    AttributeProxy_<MaybeColor>   tab_color = {"tab_color", {Inherit()}};
+    AttributeProxy_<MaybeColor>   tab_outer_color = {"tab_outer_color", {Inherit()}};
+    AttributeProxy_<MaybeULong>   tab_outer_width = {"tab_outer_width", {Inherit()}};
+    AttributeProxy_<MaybeColor>   tab_title_color = {"tab_title_color", {Inherit()}};
 
     Signal scheme_changed_; //! whenever one of the attributes changes.
 
-    Rectangle inner_rect_to_outline(Rectangle rect) const;
-    Rectangle outline_to_inner_rect(Rectangle rect) const;
+    Rectangle inner_rect_to_outline(Rectangle rect, size_t tabCount) const;
+    Rectangle outline_to_inner_rect(Rectangle rect, size_t tabCount) const;
+    bool showTitle(size_t tabCount) const;
 
     // after having called this with some vector 'decs', then if an attribute
     // is changed here, then the attribute with the same name is changed
@@ -130,15 +181,21 @@ public:
     }
 };
 
+/**
+ * @brief the members of the 'theme' object. It is a separate
+ * enum class (and not nested in Theme) because this makes
+ * forward declaration possible.
+ */
+enum class ThemeType {
+    Fullscreen,
+    Tiling,
+    Floating,
+    Minimal,
+};
+
 class Theme : public DecTriple {
 public:
-    enum class Type {
-        Fullscreen,
-        Tiling,
-        Floating,
-        Minimal,
-    };
-    const DecTriple& operator[](Type t) const {
+    const DecTriple& operator[](ThemeType t) const {
         return *decTriples[static_cast<int>(t)];
     };
     Theme();
