@@ -24,11 +24,6 @@ public:
 
     // return an XColor as obtained form XQueryColor
     XColor toXColor() const;
-    unsigned long toX11Pixel() const { return x11pixelValue_; }
-
-    static unsigned long x11PixelPlusAlpha(unsigned long x11pixel, unsigned short alpha) {
-        return (x11pixel & 0xffffffu) | (alpha << 24);
-    }
 
     bool operator==(const Color& other) const {
         return red_ == other.red_
