@@ -8,6 +8,8 @@
 #include "optional.h"
 #include "rectangle.h"
 
+class Color;
+
 class XConnection {
 private:
     XConnection(Display* disp);
@@ -26,12 +28,17 @@ public:
     int depth() { return depth_; }
     Visual* visual() { return visual_; }
 
+    unsigned long allocColor(Colormap maybeColormap, const Color& color);
+
     bool otherWmListensRoot(); // return whether another WM is running
     void tryInitTransparency();
     bool usesTransparency() { return usesTransparency_; }
+    void setCompositorRunning(bool running);
 
     // utility functions
     static const char* requestCodeToString(int requestCode);
+    static const char* focusChangedDetailToString(int focusedChangedEventDetail);
+    void setWindowUrgencyHint(Window window, bool urgent);
     Rectangle windowSize(Window window);
     int windowPid(Window window);
     int windowPgid(Window window);
@@ -53,6 +60,7 @@ public:
     void setPropertyString(Window w, Atom property, const std::vector<std::string>& value);
     void setPropertyWindow(Window w, Atom property, const std::vector<Window>& value);
     void setPropertyCardinal(Window w, Atom property, const std::vector<long>& value);
+    void deleteProperty(Window w, Atom property);
     std::experimental::optional<Window> getTransientForHint(Window win);
     std::vector<Window> queryTree(Window window);
     static void setExitOnError(bool exitOnError);
@@ -68,6 +76,7 @@ private:
     Visual* visual_;
     Colormap colormap_;
     bool usesTransparency_ = false;
+    bool compositorRunning_ = false;
     static bool     exitOnError_; //! exit on any xlib error
     static XConnection* s_connection;
 };
