@@ -32,6 +32,7 @@
 #include "utils.h"
 #include "watchers.h"
 #include "xconnection.h"
+#include "xkeygrabber.h"
 
 using std::make_pair;
 using std::function;
@@ -68,7 +69,8 @@ XMainLoop::XMainLoop(XConnection& X, Root* root)
     handlerTable_[ EnterNotify       ] = EH(&XMainLoop::enternotify);
     handlerTable_[ Expose            ] = EH(&XMainLoop::expose);
     handlerTable_[ FocusIn           ] = EH(&XMainLoop::focusin);
-    handlerTable_[ KeyPress          ] = EH(&XMainLoop::keypress);
+    handlerTable_[ KeyPress          ] = EH(&XMainLoop::keypressOrRelease);
+    handlerTable_[ KeyRelease        ] = EH(&XMainLoop::keypressOrRelease);
     handlerTable_[ MapNotify         ] = EH(&XMainLoop::mapnotify);
     handlerTable_[ MapRequest        ] = EH(&XMainLoop::maprequest);
     handlerTable_[ MappingNotify     ] = EH(&XMainLoop::mappingnotify);
@@ -532,16 +534,16 @@ void XMainLoop::focusin(XFocusChangeEvent* event) {
     duringFocusIn_ = false;
 }
 
-void XMainLoop::keypress(XKeyEvent* event) {
-    //HSDebug("name is: KeyPress\n");
-    root_->keys()->handleKeyPress(event);
+void XMainLoop::keypressOrRelease(XKeyEvent* event) {
+    // HSDebug("name is: KeyPress or KeyRelease\n");
+    root_->keys()->handleKeyComboEvent(root_->xKeyGrabber_->xEventToKeyCombo(event));
 }
 
 void XMainLoop::mappingnotify(XMappingEvent* ev) {
     // regrab when keyboard map changes
     XRefreshKeyboardMapping(ev);
     if(ev->request == MappingKeyboard) {
-        root_->keys()->regrabAll();
+        root_->xKeyGrabber_->regrabAll();
         //TODO: mouse_regrab_all();
     }
 }
