@@ -46,7 +46,7 @@ private:
     void enternotify(XCrossingEvent* ce);
     void expose(XEvent* event);
     void focusin(XFocusChangeEvent* event);
-    void keypress(XKeyEvent* event);
+    void keypressOrRelease(XKeyEvent* event);
     void mappingnotify(XMappingEvent* event);
     void motionnotify(XMotionEvent* event);
     void mapnotify(XMapEvent* event);
