@@ -826,7 +826,7 @@ class X11:
         return w, self.winid_str(w)
 
     def screenshot(self, win_handle) -> RawImage:
-        """screenshot of a windows content, not including its border"""
+        """screenshot of a window's content, not including its border"""
         geom = win_handle.get_geometry()
         attr = win_handle.get_attributes()
         # Xlib defines AllPlanes as: ((unsigned long)~0L)
@@ -860,7 +860,14 @@ class X11:
             assert raw.depth in [32, 24]
             # both for depth 32 and 24, the order is BGRA
             pixelsize = 4
-            (blue, green, red) = (0, 1, 2)
+            # actually, there should be an attribute raw.byte_order which
+            # tells the byte order in the present XImage,
+            # but this is unfortunately missing in python-xlib. so let's
+            # work around this using the system's settings:
+            if sys.byteorder == 'little':
+                (blue, green, red) = (0, 1, 2)
+            else:
+                (blue, green, red) = (3, 2, 1)
             size = geom.width * geom.height
             assert len(raw.data) == pixelsize * size
             rgbvals = [(raw.data[pixelsize * (y * geom.width + x) + red],
@@ -932,19 +939,36 @@ class X11:
         geom.y = y
         return geom
 
-    def get_hlwm_frames(self):
+    def get_hlwm_frames(self, only_visible=False):
         """get list of window handles of herbstluftwm
         frame decoration windows"""
-        cmd = ['xdotool', 'search', '--class', '_HERBST_FRAME']
+        cmd = ['xdotool', 'search']
+        if only_visible:
+            cmd += ['--onlyvisible']
+        cmd += ['--class', '_HERBST_FRAME']
         frame_wins = subprocess.run(cmd,
                                     stdout=subprocess.PIPE,
-                                    universal_newlines=True,
-                                    check=True)
+                                    universal_newlines=True)
+        # we need to ignore the exit code, because xdotool returns exit
+        # code 1 if no windows were found.
         res = []
         for winid_decimal_str in frame_wins.stdout.splitlines():
             res.append(self.window(winid_decimal_str))
         return res
 
+    def get_numlock_state(self):
+        return (self.display.get_keyboard_control().led_mask & 2) != 0
+
+    def set_numlock_state(self, enabled):
+        if self.get_numlock_state() != enabled:
+            cmd = ['xdotool', 'key', '--clearmodifiers', 'Num_Lock']
+            subprocess.run(cmd, check=True)
+            for _ in range(0, 10):
+                self.display.sync()
+                if self.get_numlock_state() == enabled:
+                    return
+            assert self.get_numlock_state() == enabled
+
     def shutdown(self):
         # Destroy all created windows:
         for window in self.windows:
@@ -1050,14 +1074,34 @@ class MultiscreenDisplay:
 def keyboard():
     class KeyBoard:
         def press(self, key_spec):
+            print(f"send key press and release {key_spec}", file=sys.stderr)
             subprocess.check_call(['xdotool', 'key', key_spec])
 
         def down(self, key_spec):
+            print(f"send key release {key_spec}", file=sys.stderr)
             subprocess.check_call(['xdotool', 'keydown', key_spec])
 
         def up(self, key_spec):
+            print(f"send key press {key_spec}", file=sys.stderr)
             subprocess.check_call(['xdotool', 'keyup', key_spec])
 
+        def get_focus(self):
+            """return the window id (hex number as string) of the
+            window that receives the keyboard events"""
+            proc = subprocess.run(['xdotool', 'getwindowfocus'], stdout=subprocess.PIPE)
+            assert proc.returncode == 0
+            return hex(int(proc.stdout))
+
+        def wait_until_window_is_focused(self, winid):
+            """wait until the given winid obtains the input focus"""
+            timeout = 20
+            while timeout > 0:
+                if self.get_focus() == winid:
+                    return
+                timeout -= 1
+                time.sleep(0.1)
+            assert self.get_focus() == winid
+
     return KeyBoard()
 
 
