@@ -1,5 +1,6 @@
 import pytest
 import random
+from conftest import PROCESS_SHUTDOWN_TIME
 from herbstluftwm.types import Rectangle
 
 
@@ -58,7 +59,7 @@ def test_close_focused_client(hlwm, running_clients, running_clients_num):
 
     hlwm.call('close')
 
-    proc.wait(20)  # wait for the client to shut down
+    proc.wait(PROCESS_SHUTDOWN_TIME)  # wait for the client to shut down
     hlwm.call('true')  # sync with hlwm
     clients = hlwm.list_children('clients')
     # all other clients still run:
@@ -75,7 +76,7 @@ def test_close_unfocused_client(hlwm):
 
     hlwm.call(['close', unfocused])
 
-    proc.wait(10)  # wait
+    proc.wait(PROCESS_SHUTDOWN_TIME)  # wait
     # the other client is still running:
     assert hlwm.attr.clients.focus.winid() == focused
 
@@ -89,7 +90,7 @@ def test_close_unmanaged_client(hlwm):
 
     hlwm.call(['close', winid])
 
-    proc.wait(10)  # wait for the client to shut down
+    proc.wait(PROCESS_SHUTDOWN_TIME)  # wait for the client to shut down
 
 
 def test_close_completion(hlwm):
@@ -440,3 +441,231 @@ def test_floating_geometry_change(hlwm, minimized, floating, x11, othertag):
         if not minimized and not othertag:
             assert (clientobj.content_geometry() == geom) == floating
             assert (x11.get_absolute_top_left(handle) == (geom.x, geom.y)) == floating
+
+
+@pytest.mark.parametrize("client_float,tag_float", [
+    (True, False), (False, True), (True, True),
+])
+def test_floating_effectively_on_attribute_change(hlwm, client_float, tag_float):
+    client, _ = hlwm.create_client()
+    clientobj = hlwm.attr.clients[client]
+    assert clientobj.floating_effectively() is False
+
+    if client_float:
+        clientobj.floating = True
+    if tag_float:
+        hlwm.attr.tags.focus.floating = True
+
+    assert clientobj.floating_effectively() is True
+
+
+def test_floating_effectively_on_tag_move(hlwm):
+    hlwm.call('add othertag')
+    hlwm.attr.tags['by-name'].othertag.floating = True
+    client, _ = hlwm.create_client()
+    clientobj = hlwm.attr.clients[client]
+    assert clientobj.floating_effectively() is False
+
+    assert clientobj.winid() == client
+    hlwm.call('move othertag')
+    assert clientobj.floating_effectively() is True
+
+    # move back to non-floating tag
+    hlwm.call(f'bring {client}')
+    assert clientobj.floating_effectively() is False
+
+
+def test_floating_effectively_on_floating_rule(hlwm):
+    hlwm.call('rule floating=on')
+    client, _ = hlwm.create_client()
+    assert hlwm.attr.clients[client].floating_effectively() is True
+
+
+def test_floating_effectively_on_tag_rule(hlwm):
+    hlwm.call('add floattag')
+    hlwm.attr.tags['by-name'].floattag.floating = True
+    hlwm.call('rule tag=floattag')
+
+    client, _ = hlwm.create_client()
+    assert hlwm.attr.clients[client].floating_effectively() is True
+
+
+def test_floating_effectively_x11_property(hlwm, x11):
+    handle, winid = x11.create_client()
+
+    for client_float in [True, False]:
+        for tag_float in [True, False]:
+            hlwm.attr.clients[winid].floating = client_float
+            hlwm.attr.tags.focus.floating = tag_float
+            x11.sync_with_hlwm()
+            prop_float = x11.get_property('HLWM_FLOATING_WINDOW', handle)
+            prop_tile = x11.get_property('HLWM_TILING_WINDOW', handle)
+
+            if client_float or tag_float:
+                assert hlwm.attr.clients[winid].floating_effectively() is True
+                assert prop_float[0] == 1
+                assert prop_tile is None
+            else:
+                assert hlwm.attr.clients[winid].floating_effectively() is False
+                assert prop_float is None
+                assert prop_tile[0] == 1
+
+
+def assert_decoration_state_correct(hlwm, x11, winid):
+    handle = x11.window(winid)
+    client_obj = hlwm.attr.clients[winid]
+    assert client_obj.content_geometry() \
+        == x11.get_absolute_geometry(handle)
+    tree = handle.query_tree()
+    if client_obj.decorated():
+        # assume no-zero border:
+        assert client_obj.content_geometry() != client_obj.decoration_geometry()
+        assert tree.root != tree.parent
+    else:
+        # for undecorated
+        assert client_obj.content_geometry() == client_obj.decoration_geometry()
+        assert tree.root == tree.parent
+
+    assert client_obj.visible() \
+        == (handle.get_attributes().map_state == 2)
+
+
+def test_decorated_off_vs_x11(hlwm, x11):
+    _, winid = x11.create_client()
+    hlwm.attr.theme.border_width = 11
+    for floating in [True, False]:
+        hlwm.attr.clients[winid].floating = floating
+        for decorated in [True, False]:
+            hlwm.attr.clients[winid].decorated = decorated
+
+            assert_decoration_state_correct(hlwm, x11, winid)
+
+
+@pytest.mark.parametrize("decorated", [True, False])
+def test_decorated_after_tag_change(hlwm, x11, decorated):
+    _, winid = x11.create_client()
+    hlwm.attr.theme.border_width = 11
+    hlwm.attr.clients[winid].decorated = decorated
+
+    assert_decoration_state_correct(hlwm, x11, winid)
+
+    # switch to other tag
+    hlwm.call('add othertag')
+    hlwm.call('use othertag')
+    assert hlwm.attr.clients[winid].visible() is False
+    assert_decoration_state_correct(hlwm, x11, winid)
+
+    # switch back to visible tag
+    hlwm.call('use_previous')
+    assert hlwm.attr.clients[winid].visible() is True
+    assert_decoration_state_correct(hlwm, x11, winid)
+
+
+def test_decorated_change_while_minimized(hlwm, x11):
+    _, winid = x11.create_client()
+    hlwm.attr.theme.border_width = 11
+    client_obj = hlwm.attr.clients[winid]
+    client_obj.decorated = False
+    sequence = [
+        # change decoration on -> off while minimized
+        ("decorated", True),
+        ("minimized", True),
+        ("decorated", False),
+        ("minimized", False),
+
+        # change decoration off -> on while minimized
+        ("decorated", False),
+        ("minimized", False),
+        ("minimized", True),
+        ("decorated", True),
+        ("minimized", False),
+    ]
+
+    for attribute, value in sequence:
+        client_obj[attribute] = value
+        assert hlwm.attr.clients[winid].visible() is not client_obj['minimized']
+        assert_decoration_state_correct(hlwm, x11, winid)
+
+
+def test_decorated_off_floating_geometry_correct(hlwm):
+    winid, _ = hlwm.create_client()
+    hlwm.attr.theme.border_width = 8
+    client_obj = hlwm.attr.clients[winid]
+
+    client_obj.floating_geometry = Rectangle(10, 20, 200, 100)
+    client_obj.floating = True
+    client_obj.decorated = False
+
+    assert client_obj.floating_geometry() == client_obj.decoration_geometry()
+
+
+@pytest.mark.parametrize("other_client", [True, False])
+def test_urgent_attribute_triggers_tag_status(hlwm, hc_idle, other_client):
+    if other_client:
+        hlwm.create_client()
+
+    hlwm.call('add othertag')
+    hlwm.call('rule tag=othertag')
+
+    winid, _ = hlwm.create_client()
+
+    # the new client is not focused:
+    assert 'focus' not in hlwm.list_children('clients') \
+        or hlwm.attr.clients.focus.winid() != winid
+
+    hc_idle.hooks()  # reset hooks
+    hlwm.attr.clients[winid].urgent = True  # make it urgent
+    assert ['tag_flags'] in hc_idle.hooks()
+    assert hlwm.attr.clients[winid].urgent() is True
+
+    hlwm.call('jumpto urgent')
+    assert hlwm.attr.clients[winid].urgent() is False
+    assert hlwm.attr.clients.focus.winid() == winid
+
+
+def test_focused_client_is_not_urgent(hlwm):
+    winid, _ = hlwm.create_client()
+    hlwm.attr.clients.focus.urgent = True
+    assert hlwm.attr.clients.focus.urgent() is False
+
+
+def test_urgent_attribute_written_to_x11(hlwm, x11):
+    hlwm.call('add othertag')
+    hlwm.call('rule tag=othertag')
+    handle, winid = x11.create_client()
+
+    assert hlwm.attr.clients[winid].urgent() is False
+    for value in [True, False, True, False]:
+        hlwm.attr.clients[winid].urgent = value
+
+        assert x11.is_window_urgent(handle) == value
+
+
+def test_unmanaged_client_is_removed_from_tab_bars(hlwm, x11):
+    hlwm.call(['add', 'othertag'])
+    hlwm.call('use_index 1')
+    hlwm.attr.theme.title_height = 14
+    hlwm.attr.theme.title_color = 'red'
+    hlwm.attr.settings.tabbed_max = True
+
+    # put two clients on 'othertag' in tabbed mode
+    # keep the tag visible such that the tab bar is filled.
+    winid_kill, proc_kill = hlwm.create_client()
+    tab_handle, tab_winid = x11.create_client()
+    assert winid_kill in hlwm.list_children('clients')
+    assert hlwm.attr.tags['1'].name() == 'othertag'
+    assert hlwm.attr.tags['1'].client_count() == 2
+    hlwm.attr.tags['1'].tiling.focused_frame.algorithm = 'max'
+    hlwm.attr.tags['1'].tiling.focused_frame.selection = 0
+    # make the tag invisible
+    hlwm.call('use_index 0')
+
+    # first, kill one of the clients
+    proc_kill.terminate()
+    proc_kill.wait()
+    # then, trigger a repaint of the tab bar of the remaining window:
+    x11.set_window_title(tab_handle, "some other title")
+    # (earlier, this crashed because 'proc_kill' was still mentioned in the tab
+    # bar of tab_handle/tab_winid)
+
+    assert tab_winid in hlwm.list_children('clients')
