@@ -232,15 +232,16 @@ def test_remove_attr(hlwm):
 
 @pytest.mark.parametrize('command_prefix', [
     ['substitute', 'ARG', 'tags.count'],
-    ['foreach', 'ARG', 'clients']
+    ['foreach', 'ARG', 'clients', '--filter-name=X']
 ])
 def test_metacommand(hlwm, command_prefix):
     """test the completion of a command that accepts another
     command as the parameter
     """
     cmdlist = hlwm.call('list_commands').stdout.splitlines()
-    assert hlwm.complete(command_prefix) \
-        == sorted(['ARG'] + cmdlist)
+    # filter all flags out of the completions:
+    completions = [cmd for cmd in hlwm.complete(command_prefix) if cmd[0] != '-']
+    assert completions == sorted(['ARG'] + cmdlist)
 
 
 def test_posix_escape_via_use(hlwm):
