@@ -1,4 +1,5 @@
 import pytest
+import re
 from test_layout import verify_frame_objects_via_dump
 
 
@@ -96,9 +97,10 @@ def test_full_layouts(hlwm, layout):
     "(clients vertical:0 1713)",
 ])
 def test_load_invalid_winids(hlwm, layout):
-    p = hlwm.call(['load', layout])
+    message = 'load: Warning: Unknown window IDs'
+    p = hlwm.call(['load', layout], allowed_stderr=re.compile(message)) \
 
-    assert p.stdout.startswith("Warning: Unknown window IDs")
+    assert p.stderr.startswith(message)
 
 
 @pytest.mark.parametrize(
@@ -210,3 +212,13 @@ def test_load_minimized_client(hlwm, othertag, minimized, floating):
     assert hlwm.get_attr(f'clients.{winid}.visible') == 'true'
     assert hlwm.get_attr(f'clients.{winid}.minimized') == 'false'
     assert hlwm.get_attr(f'clients.{winid}.floating') == 'false'
+
+
+def test_load_completion(hlwm):
+    tagname = 'newtag'
+    hlwm.call(f'add {tagname}')
+
+    assert tagname in hlwm.complete(['load'])
+    assert tagname not in hlwm.complete(['load', tagname])
+    hlwm.command_has_all_args(['load', '(clients ...)'])
+    hlwm.command_has_all_args(['load', tagname, 'bar'])
