@@ -1,4 +1,5 @@
 import pytest
+from herbstluftwm.types import Rectangle
 from Xlib import Xatom
 
 
@@ -41,6 +42,20 @@ def test_auto_detect_panels(hlwm, x11, value):
         hlwm.attr.settings.auto_detect_panels = 'toggle'
 
 
+def test_panel_object(hlwm, x11):
+    assert int(hlwm.attr.panels.count()) == 0
+
+    _, winid = x11.create_client(geometry=(1, 0, 800, 30),
+                                 wm_class=('panelinst', 'panelclass'),
+                                 window_type='_NET_WM_WINDOW_TYPE_DOCK')
+
+    assert int(hlwm.attr.panels.count()) == 1
+    assert hlwm.attr.panels[winid].instance() == 'panelinst'
+    assert hlwm.attr.panels[winid]['class']() == 'panelclass'
+    assert hlwm.attr.panels[winid].geometry() == Rectangle(1, 0, 800, 30)
+    assert hlwm.attr.panels[winid].winid() == winid
+
+
 @pytest.mark.parametrize("which_pad, pad_size, geometry", [
     ("pad_left", 10, (-1, 0, 11, 400)),
     ("pad_right", 20, (800 - 20, 23, 20, 400)),
@@ -59,11 +74,11 @@ def test_panel_based_on_intersection(hlwm, x11, which_pad, pad_size, geometry):
 
     assert int(hlwm.attr.monitors[0][which_pad]()) == pad_size
     for p in ["pad_left", "pad_right", "pad_up", "pad_down"]:
-        assert hlwm.attr.monitors[1][p]() == '0', \
+        assert hlwm.attr.monitors[1][p]() == 0, \
             "monitor 1 must never be affected"
         if p == which_pad:
             continue
-        assert hlwm.attr.monitors[0][p]() == '0'
+        assert hlwm.attr.monitors[0][p]() == 0
 
 
 @pytest.mark.parametrize("which_pad, pad_size, geometry, wm_strut", [
@@ -97,11 +112,11 @@ def test_panel_based_on_wmstrut(hlwm, x11, which_pad, pad_size, geometry, wm_str
 
     assert int(hlwm.attr.monitors[0][which_pad]()) == pad_size
     for p in ["pad_left", "pad_right", "pad_up", "pad_down"]:
-        assert hlwm.attr.monitors[1][p]() == '0', \
+        assert hlwm.attr.monitors[1][p]() == 0, \
             "monitor 1 must never be affected"
         if p == which_pad:
             continue
-        assert hlwm.attr.monitors[0][p]() == '0'
+        assert hlwm.attr.monitors[0][p]() == 0
 
 
 def test_panel_wm_strut_partial_on_big_screen(hlwm, x11):
@@ -128,7 +143,7 @@ def test_panel_wm_strut_partial_on_big_screen(hlwm, x11):
     assert hlwm.call('list_padding').stdout.strip() == '54 0 0 0'
 
 
-@pytest.mark.parametrize("xvfb", [(1280 + 1024, 1024)], indirect=True)
+@pytest.mark.parametrize("xvfb", [{'resolution': (1280 + 1024, 1024)}], indirect=True)
 def test_panel_wm_strut_partial_different_sized_screens(hlwm, x11):
     """This is the xinerama example from the EWMH doc on _NET_WM_STRUT_PARTIAL
     https://specifications.freedesktop.org/wm-spec/wm-spec-latest.html#idm45075509080400
@@ -162,7 +177,7 @@ def test_panel_wm_strut_partial_different_sized_screens(hlwm, x11):
     assert hlwm.call('list_padding 1').stdout.strip() == '0 0 50 0'
 
 
-@pytest.mark.parametrize("xvfb", [(800 + 700, 600)], indirect=True)
+@pytest.mark.parametrize("xvfb", [{'resolution': (800 + 700, 600)}], indirect=True)
 def test_panel_wm_strut_between_monitors(hlwm, x11):
     hlwm.call('add anothertag')
     hlwm.call('set_monitors 800x600+0+0 700x600+800+0')
@@ -187,7 +202,7 @@ def test_panel_wm_strut_between_monitors(hlwm, x11):
 
 
 @pytest.mark.parametrize("edge", ["left", "right", "up", "down"])
-@pytest.mark.parametrize("xvfb", [(2 * 800, 2 * 600)], indirect=True)
+@pytest.mark.parametrize("xvfb", [{'resolution': (2 * 800, 2 * 600)}], indirect=True)
 def test_panel_wm_strut_between_monitors_2x2_grid(hlwm, x11, edge):
     hlwm.call('chain , add t1 , add t2 , add t3')
     hlwm.call(['set_monitors',
