@@ -9,9 +9,13 @@ def test_spawn(hlwm, hlwm_process):
 
 def test_spawn_command_not_exist(hlwm, hlwm_process):
     cmdname = 'this_command_does_not_exist'
-    with hlwm_process.wait_stderr_match(f'execvp "{cmdname}" failed'):
-        cmd = ['spawn', cmdname]
-        proc = hlwm.unchecked_call(cmd, read_hlwm_output=False)
-        assert proc.returncode == 0
-        assert not proc.stderr
-        assert not proc.stdout
+    hlwm.call_xfail(['spawn', cmdname]) \
+        .expect_stderr(cmdname) \
+        .expect_stderr('No such file')
+
+
+def test_spawn_command_no_permission(hlwm, tmpdir, hlwm_process):
+    dirname = str(tmpdir)
+    hlwm.call_xfail(['spawn', dirname]) \
+        .expect_stderr(dirname) \
+        .expect_stderr('Permission denied')
